@@ -113,6 +113,8 @@ class NWMElementPairings:
             raise IOError(
                 "No National Water model intersections found on the mesh.")
         intersection = gpd.GeoDataFrame(data, crs=hgrid.crs)
+        #TODO: add exporting intersection as an option
+        #intersection.to_file('intersections.shp')
         del data
 
         # 2) Generate element centroid KDTree
@@ -316,7 +318,26 @@ class NWMElementPairings:
                         f"Extracting National Water Model stream network tar file to {DATADIR}"
                     )
 
-                    src.extractall(DATADIR)
+                    def is_within_directory(directory, target):
+                        
+                        abs_directory = os.path.abspath(directory)
+                        abs_target = os.path.abspath(target)
+                    
+                        prefix = os.path.commonprefix([abs_directory, abs_target])
+                        
+                        return prefix == abs_directory
+                    
+                    def safe_extract(tar, path=".", members=None, *, numeric_owner=False):
+                    
+                        for member in tar.getmembers():
+                            member_path = os.path.join(path, member.name)
+                            if not is_within_directory(path, member_path):
+                                raise Exception("Attempted Path Traversal in Tar File")
+                    
+                        tar.extractall(path, members, numeric_owner=numeric_owner)
+                        
+                    
+                    safe_extract(src, DATADIR)
                 nwm_file = list(DATADIR.glob("**/*.gdb"))[0]
             elif len(nwm_file) == 1:
                 nwm_file = nwm_file[0]
@@ -370,12 +391,14 @@ class AWSDataInventory(ABC):
             return AWSHindcastInventory.__new__(cls)
 
         # GOOGLEHindcastInventory -> January 2019 through 30 days earlier than today
+        # data before April 20, 2021 (including Apirl 20) is 3-hr interval, after that is hourly
+        # Missing data 20211231, 20220101, 20220102, 20220103
         elif start_date >= dates.localize_datetime(
             datetime(2021, 1, 1, 0, 0)
-        ) and start_date + rnday < dates.nearest_zulu() - timedelta(days=30):
+        ) and start_date + rnday < dates.nearest_zulu() - timedelta(days=2):
             return GOOGLEHindcastInventory.__new__(cls)
 
-        elif start_date >= dates.nearest_zulu() - timedelta(days=30):
+        elif start_date >= dates.nearest_zulu() - timedelta(days=2):
             return AWSForecastInventory.__new__(cls)
 
         else:
@@ -390,7 +413,7 @@ class AWSDataInventory(ABC):
     #@property
     #@abstractmethod
     #def bucket(self):
-        raise NotImplementedError
+    #    raise NotImplementedError
 
     @property
     def nearest_cycle(self) -> datetime:
@@ -561,7 +584,7 @@ class GOOGLEHindcastInventory(AWSDataInventory):
         self,
         start_date: datetime = None,
         rnday: Union[int, float, timedelta] = timedelta(days=5.0),
-        product='analysis_assim.channel_rt',
+        product='medium_range_mem1',
         verbose=False,
         fallback=True,
         cache=None,
@@ -569,7 +592,6 @@ class GOOGLEHindcastInventory(AWSDataInventory):
 
         self.start_date = dates.nearest_cycle() if start_date is None \
             else dates.nearest_cycle(dates.localize_datetime(start_date))
-        # self.start_date = self.start_date.replace(tzinfo=None)
         self.rnday = rnday if isinstance(rnday, timedelta) \
             else timedelta(days=rnday)
         self.product = product
@@ -588,37 +610,54 @@ class GOOGLEHindcastInventory(AWSDataInventory):
             dtype='datetime64')
 
         for requested_time, _ in self._files.items():
-            #print(f'Requesting NWM data for time {requested_time}')
-            print(f'Requesting NWM data for time {requested_time}')
+
             logger.info(f'Requesting NWM data for time {requested_time}')
 
-            self._files[requested_time] = self.request_data(requested_time)
+            if requested_time.hour == 0:
+                requested_time2 = requested_time - timedelta(days=1)
+            else:
+                requested_time2 = requested_time
 
-    def request_data(self, request_time):
+            self._files[requested_time] = self.request_data(requested_time, requested_time2)
+
+    def request_data(self, request_time, request_time2):
+
+        fname = self.tmpdir / f'nwm.t00z.{self.product[:12]}.channel_rt_1.' \
+            f'{request_time.strftime("%Y%m%d%H")}.conus.nc'
+        #logger.info(f'fname is {fname}')
 
-        fname = self.tmpdir / f'{request_time.strftime("%Y%m%d%H")}00.CHRTOUT_DOMAIN1.comp'
         if fname.is_file():
             cached_file = list(self.tmpdir.glob(f'**/{fname.name}'))
             if len(cached_file) == 1:
                 fname = cached_file[0]
                 logger.info(f'Use cached file {fname}')
-            else:
-          
-                fname = f'./analysis_assim/{request_time.strftime("%Y%m%d%H")}00.CHRTOUT_DOMAIN1.comp'
+        else:
+        
+            fname = f'{self.start_date.strftime("%Y%m%d")}/nwm.t00z.' \
+                f'{self.product[:12]}.channel_rt_1.{request_time.strftime("%Y%m%d%H")}.conus.nc'
 
-                logger.info(f'Downloading file {request_time}, ')
-        #nwm.20180917/forcing_medium_range/nwm.t12z.medium_range.forcing.f160.conus.nc
-        #https://storage.googleapis.com/national-water-model/nwm.20210409/analysis_assim/nwm.t00z.analysis_assim.channel_rt.tm01.conus.nc
-                url=f'https://storage.googleapis.com/national-water-model/nwm.{request_time.strftime("%Y%m%d")}' \
-                    f'/analysis_assim/nwm.t{request_time.strftime("%H")}z.analysis_assim.channel_rt.tm00.conus.nc'
-                logger.info(f'{url}')
+            logger.info(f'Downloading file {request_time}, ')
+
+            it = request_time.strftime("%H")
+            if it == '00':
+                logger.info(f'Requesting data at 00Z from yesterday!')
+                it = str(int(it) + 24)
+            it = it.zfill(3)
+
+            url = f'https://storage.googleapis.com/national-water-model/nwm.{request_time2.strftime("%Y%m%d")}' \
+                f'/{self.product}/nwm.t00z.{self.product[:12]}.channel_rt_1.f{it}.conus.nc'
+            logger.info(f'{url}')
+            try:
                 wget.download(url, fname)
+            except:
+                logger.info(f'No data for {request_time}!')
+                
         return fname
 
     @property
     def output_interval(self) -> timedelta:
         return {
-            'analysis_assim.channel_rt': timedelta(hours=1)
+            'medium_range_mem1': timedelta(hours=1)
         }[self.product]
 
     @property
@@ -675,6 +714,16 @@ class AWSForecastInventory(AWSDataInventory):
             if start_date is None
             else dates.nearest_cycle(dates.localize_datetime(start_date))
         )
+
+        yesterday = self.start_date - timedelta(days=1)
+       
+        #check if previous day's data folder exists
+        if not os.path.exists(yesterday.strftime("%Y%m%d")):
+            logger.info(f"Downloading NWM data for {yesterday}")
+            _ = self.download(yesterday, days=1)
+        
+        filemaps = self.download(self.start_date, days=rnday.days)
+
         # self.start_date = self.start_date.replace(tzinfo=None)
         self.rnday = rnday if isinstance(
             rnday, timedelta) else timedelta(days=rnday)
@@ -687,26 +736,29 @@ class AWSForecastInventory(AWSDataInventory):
                 self.output_interval,
             ).astype(datetime)
         }
+
+        for it, (requested_time, _) in enumerate(self._files.items()):
+
+            logger.info(f"Requesting NWM data for time {requested_time}")
+            if it == 0 and requested_time.hour == 0:
+                f0 = f'{yesterday.strftime("%Y%m%d")}/nwm.' \
+                    + yesterday.strftime("%Y%m%d") \
+                    + '/medium_range_mem1/nwm.t00z.medium_range.channel_rt_1.f024.conus.nc'
+                logger.info(f'Using data {f0}')
+                self.files[requested_time] = f0
+                continue
+
+            logger.info(f"Using data {filemaps[requested_time]}")
+            self._files[requested_time] = filemaps[requested_time]
+
+    def download(self, nwmdate, days):
         self.data = self.s3.list_objects_v2(
             Bucket=self.bucket,
             Delimiter="/",
-            Prefix=f'nwm.{self.start_date.strftime("%Y%m%d")}' f"/{self.product}/",
+            Prefix=f'nwm.{nwmdate.strftime("%Y%m%d")}' f"/{self.product}/",
         )
 
-        for it, (requested_time, _) in enumerate(self._files.items()):
-            if it == 0 and requested_time.hour == 0:
-                logger.info(f'set it=0 {requested_time}')
-                yesterday = (self.start_date - timedelta(days=1)).strftime("%Y%m%d")
-                self.files[requested_time] = f'{yesterday}/nwm.{yesterday}/medium_range' \
-                + '_mem1/nwm.t00z.medium_range.channel_rt_1.f024.conus.nc'
-                continue
-            logger.info(f"Requesting NWM data for time {requested_time}")
-            self._files[requested_time] = self.request_data(requested_time)
-        
-    def request_data(self, request_time):
-
         file_metadata = list(
-#            reversed(
                 sorted(
                     [
                         _["Key"]
@@ -715,20 +767,22 @@ class AWSForecastInventory(AWSDataInventory):
                     ]
                 )
             )
-#        )
 
-        for key in file_metadata[0:240]:
-            if request_time != self.key2date(key):
-                continue
-            filename = self.tmpdir / key
-            filename.parent.mkdir(parents=True, exist_ok=True)
-            if filename.is_file() is False:
-                tmpfile = tempfile.NamedTemporaryFile().name
-                with open(tmpfile, "wb") as f:
-                    logger.info(f"Downloading file {key}, ")
-                    self.s3.download_fileobj(self.bucket, key, f)
-                shutil.move(tmpfile, filename)
-            return filename
+        filedir = pathlib.Path(nwmdate.strftime("%Y%m%d"))
+        filedir.mkdir(exist_ok=True, parents=True)
+
+        filedict = {}
+        for key in file_metadata[0:days*24+1]:
+            filetime = self.key2date(key)
+            filename = nwmdate.strftime("%Y%m%d") + '/' + key
+            filesubdir = pathlib.Path(filename)
+            filesubdir.parent.mkdir(parents=True, exist_ok=True)
+
+            logger.info(f"Downloading file {key}, ")
+            self.s3.download_file(self.bucket, key, filename)
+            filedict[filetime] = filename
+       
+        return filedict
 
     def key2date(self, key):
         base_date_str = f'{key.split("/")[0].split(".")[-1]}'
@@ -755,7 +809,7 @@ class AWSForecastInventory(AWSDataInventory):
 
     @property
     def output_interval(self) -> timedelta:
-        return {"medium_range_mem1": timedelta(hours=3)}[self.product]
+        return {"medium_range_mem1": timedelta(hours=1)}[self.product]
 
     @property
     def timevector(self):
